@@ -1,4 +1,4 @@
-# Copyright (c) 2016-2019 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2016-2021 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -42,8 +42,6 @@ except ImportError as exc: # pragma: no cover
         raise OSError(errno.ENOENT, 'Agent support unavailable: %s' % reason)
 
 
-# pylint: disable=bad-whitespace
-
 # Client request message numbers
 SSH_AGENTC_REQUEST_IDENTITIES            = 11
 SSH_AGENTC_SIGN_REQUEST                  = 13
@@ -74,8 +72,6 @@ SSH_AGENT_CONSTRAIN_EXTENSION            = 255
 SSH_AGENT_RSA_SHA2_256                   = 2
 SSH_AGENT_RSA_SHA2_512                   = 4
 
-# pylint: enable=bad-whitespace
-
 
 class SSHAgentKeyPair(SSHKeyPair):
     """Surrogate for a key managed by the SSH agent"""
@@ -83,58 +79,46 @@ class SSHAgentKeyPair(SSHKeyPair):
     _key_type = 'agent'
 
     def __init__(self, agent, algorithm, public_data, comment):
-        super().__init__(algorithm, public_data, comment)
+        is_cert = algorithm.endswith(b'-cert-v01@openssh.com')
 
-        self._agent = agent
-        self.sig_algorithm = algorithm
-        self.sig_algorithms = (algorithm,)
-        self.host_key_algorithms = self.sig_algorithms
-
-        self._cert = algorithm.endswith(b'-cert-v01@openssh.com')
-        self._flags = 0
-
-        self._set_algorithms()
-
-    def _set_algorithms(self):
-        """Set the algorithms associated with this key"""
-
-        if self._cert:
-            if self.algorithm.startswith(b'sk-'):
-                self.sig_algorithm = self.algorithm[:-21] + b'@openssh.com'
+        if is_cert:
+            if algorithm.startswith(b'sk-'):
+                sig_algorithm = algorithm[:-21] + b'@openssh.com'
             else:
-                self.sig_algorithm = self.algorithm[:-21]
+                sig_algorithm = algorithm[:-21]
+        else:
+            sig_algorithm = algorithm
 
         # Neither Pageant nor the Win10 OpenSSH agent seems to support the
         # ssh-agent protocol flags used to request RSA SHA2 signatures yet
-        if self.sig_algorithm == b'ssh-rsa' and sys.platform != 'win32':
-            self.sig_algorithms = (b'rsa-sha2-256', b'rsa-sha2-512',
-                                   b'ssh-rsa')
+        if sig_algorithm == b'ssh-rsa' and sys.platform != 'win32':
+            sig_algorithms = (b'rsa-sha2-256', b'rsa-sha2-512', b'ssh-rsa')
         else:
-            self.sig_algorithms = (self.sig_algorithm,)
+            sig_algorithms = (sig_algorithm,)
 
-        if self._cert:
-            self.host_key_algorithms = (self.algorithm,)
+        if is_cert:
+            host_key_algorithms = (algorithm,)
         else:
-            self.host_key_algorithms = self.sig_algorithms
+            host_key_algorithms = sig_algorithms
+
+        super().__init__(algorithm, sig_algorithm, sig_algorithms,
+                         host_key_algorithms, public_data, comment)
+
+        self._agent = agent
+        self._is_cert = is_cert
+        self._flags = 0
 
     def set_certificate(self, cert):
         """Set certificate to use with this key"""
 
-        if self.public_data != cert.key.public_data:
-            raise ValueError('Certificate key mismatch')
+        super().set_certificate(cert)
 
-        self._cert = True
-        self.algorithm = cert.algorithm
-        self.public_data = cert.public_data
-        self._set_algorithms()
+        self._is_cert = True
 
     def set_sig_algorithm(self, sig_algorithm):
         """Set the signature algorithm to use when signing data"""
 
-        self.sig_algorithm = sig_algorithm
-
-        if not self._cert:
-            self.algorithm = sig_algorithm
+        super().set_sig_algorithm(sig_algorithm)
 
         if sig_algorithm == b'rsa-sha2-256':
             self._flags |= SSH_AGENT_RSA_SHA2_256
@@ -144,7 +128,7 @@ class SSHAgentKeyPair(SSHKeyPair):
     async def sign(self, data):
         """Sign a block of data with this private key"""
 
-        return await self._agent.sign(self.public_data, data, self._flags)
+        return await self._agent.sign(self.key_public_data, data, self._flags)
 
     async def remove(self):
         """Remove this key pair from the agent"""
@@ -224,12 +208,16 @@ class SSHAgentClient:
                 await self._cleanup()
                 raise ValueError(str(exc)) from None
 
-    async def get_keys(self):
+    async def get_keys(self, identities=()):
         """Request the available client keys
 
            This method is a coroutine which returns a list of client keys
            available in the ssh-agent.
 
+           :param identities: (optional)
+               A list of allowed byte string identities to return. If empty,
+               all identities on the SSH agent will be returned.
+
            :returns: A list of :class:`SSHKeyPair` objects
 
         """
@@ -245,6 +233,9 @@ class SSHAgentClient:
                 key_blob = resp.get_string()
                 comment = resp.get_string()
 
+                if identities and key_blob not in identities:
+                    continue
+
                 packet = SSHPacket(key_blob)
                 algorithm = packet.get_string()
 
@@ -523,7 +514,7 @@ class SSHAgentClient:
                 try:
                     exttype = exttype.decode('utf-8')
                 except UnicodeDecodeError:
-                    raise ValueError('Invalid extension type name')
+                    raise ValueError('Invalid extension type name') from None
 
                 result.append(exttype)
 
