@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2019 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2013-2021 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -31,8 +31,6 @@ from .packet import Boolean, String, UInt32, SSHPacketHandler
 from .saslprep import saslprep, SASLPrepError
 
 
-# pylint: disable=bad-whitespace
-
 # SSH message values for GSS auth
 MSG_USERAUTH_GSSAPI_RESPONSE          = 60
 MSG_USERAUTH_GSSAPI_TOKEN             = 61
@@ -51,8 +49,6 @@ MSG_USERAUTH_INFO_RESPONSE            = 61
 # SSH message values for password auth
 MSG_USERAUTH_PASSWD_CHANGEREQ         = 60
 
-# pylint: enable=bad-whitespace
-
 _auth_methods = []
 _client_auth_handlers = {}
 _server_auth_handlers = {}
@@ -892,6 +888,13 @@ def register_auth_method(alg, client_handler, server_handler):
     _server_auth_handlers[alg] = server_handler
 
 
+def get_client_auth_methods():
+    """Return a list of supported client auth methods"""
+
+    return [method for method in _client_auth_handlers
+            if method != b'none']
+
+
 def lookup_client_auth(conn, method):
     """Look up the client authentication method to use"""
 
@@ -902,7 +905,7 @@ def lookup_client_auth(conn, method):
 
 
 def get_server_auth_methods(conn):
-    """Return a list of supported auth methods"""
+    """Return a list of supported server auth methods"""
 
     auth_methods = []
 
@@ -925,8 +928,6 @@ def lookup_server_auth(conn, username, method, packet):
         return None
 
 
-# pylint: disable=bad-whitespace
-
 _auth_method_list = (
     (b'none',                 _ClientNullAuth,      _ServerNullAuth),
     (b'gssapi-keyex',         _ClientGSSKexAuth,    _ServerGSSKexAuth),
@@ -937,7 +938,5 @@ _auth_method_list = (
     (b'password',             _ClientPasswordAuth,  _ServerPasswordAuth)
 )
 
-# pylint: enable=bad-whitespace
-
 for _args in _auth_method_list:
     register_auth_method(*_args)
