@@ -1,4 +1,4 @@
-# Copyright (c) 2020 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2020-2021 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -285,6 +285,17 @@ class SSHConfig:
                     self._error(str(exc))
 
                 option = args.pop(0)
+
+                if option.endswith('='):
+                    option = option[:-1]
+                elif '=' in option:
+                    option, arg = option.split('=', 1)
+                    args[:0] =[arg]
+                elif args and args[0] == '=':
+                    del args[0]
+                elif args and args[0].startswith('='):
+                    args[0] = args[0][1:]
+
                 loption = option.lower()
 
                 if loption in self._no_split:
@@ -316,7 +327,7 @@ class SSHConfig:
             else:
                 if isinstance(value, list):
                     value = [self._expand_val(item) for item in value]
-                else:
+                elif isinstance(value, str):
                     value = self._expand_val(value)
 
                 self._options[option] = value
@@ -365,7 +376,7 @@ class SSHClientConfig(SSHConfig):
     _conditionals = {'host', 'match'}
     _no_split = {'remotecommand'}
     _percent_expand = {'CertificateFile', 'IdentityAgent',
-                       'IdentityFile', 'RemoteCommand'}
+                       'IdentityFile', 'ProxyCommand', 'RemoteCommand'}
 
     def _init_options(self, local_user, user, host, port):
         """Set options used for matching and token substitutions"""
@@ -456,8 +467,6 @@ class SSHClientConfig(SSHConfig):
         if hasattr(os, 'getuid'): # pragma: no branch
             self._tokens['i'] = str(os.getuid())
 
-    # pylint: disable=bad-whitespace
-
     _handlers = {option.lower(): (option, handler) for option, handler in (
         ('Host',                            _match_host),
         ('Match',                           SSHConfig._match),
@@ -481,6 +490,8 @@ class SSHClientConfig(SSHConfig):
         ('HostbasedAuthentication',         SSHConfig._set_bool),
         ('HostKeyAlgorithms',               SSHConfig._set_string),
         ('Hostname',                        _set_hostname),
+        ('HostKeyAlias',                    SSHConfig._set_string),
+        ('IdentitiesOnly',                  SSHConfig._set_bool),
         ('IdentityAgent',                   SSHConfig._set_string),
         ('IdentityFile',                    SSHConfig._append_string),
         ('KbdInteractiveAuthentication',    SSHConfig._set_bool),
@@ -490,6 +501,7 @@ class SSHClientConfig(SSHConfig):
         ('PKCS11Provider',                  SSHConfig._set_string),
         ('PreferredAuthentications',        SSHConfig._set_string),
         ('Port',                            SSHConfig._set_int),
+        ('ProxyCommand',                    SSHConfig._set_string_list),
         ('ProxyJump',                       SSHConfig._set_string),
         ('PubkeyAuthentication',            SSHConfig._set_bool),
         ('RekeyLimit',                      SSHConfig._set_rekey_limits),
@@ -540,8 +552,6 @@ class SSHServerConfig(SSHConfig):
 
         self._tokens.update({'u': self._user})
 
-    # pylint: disable=bad-whitespace
-
     _handlers = {option.lower(): (option, handler) for option, handler in (
         ('Match',                           SSHConfig._match),
         ('Include',                         SSHConfig._include),
