@@ -1,4 +1,4 @@
-# Copyright (c) 2016-2019 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2016-2020 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -23,7 +23,9 @@
 import asyncio
 from copy import copy
 import os
+from pathlib import Path
 import socket
+import subprocess
 import sys
 import unittest
 from unittest.mock import patch
@@ -46,10 +48,17 @@ from asyncssh.packet import Boolean, NameList, String, UInt32
 
 from .server import Server, ServerTestCase
 
-from .util import asynctest, gss_available, patch_gss
+from .util import asynctest, gss_available, patch_gss, run
 from .util import patch_getnameinfo, x509_available
 
 
+try:
+    run('which nc')
+    _nc_available = True
+except subprocess.CalledProcessError: # pragma: no cover
+    _nc_available = False
+
+
 class _CheckAlgsClientConnection(asyncssh.SSHClientConnection):
     """Test specification of encryption algorithms"""
 
@@ -508,6 +517,28 @@ class _TestConnection(ServerTestCase):
         with self.assertRaises(OSError):
             await asyncssh.get_server_host_key('0.0.0.1')
 
+    @unittest.skipUnless(_nc_available, 'Netcat not available')
+    @asynctest
+    async def test_get_server_host_key_proxy(self):
+        """Test retrieving a server host key using proxy command"""
+
+        keylist = asyncssh.load_public_keys('skey.pub')
+        proxy_command = ('nc', str(self._server_addr), str(self._server_port))
+
+        key = await self.get_server_host_key(proxy_command=proxy_command)
+
+        self.assertEqual(key, keylist[0])
+
+    @unittest.skipUnless(_nc_available, 'Netcat not available')
+    @asynctest
+    async def test_get_server_host_key_proxy_failure(self):
+        """Test failure retrieving a server host key using proxy command"""
+
+        proxy_command = 'nc 0.0.0.1 1'
+
+        with self.assertRaises((OSError, asyncssh.ConnectionLost)):
+            await self.connect(proxy_command=proxy_command)
+
     @asynctest
     async def test_known_hosts_not_present(self):
         """Test connecting with default known hosts file not present"""
@@ -699,6 +730,20 @@ class _TestConnection(ServerTestCase):
             with self.assertRaises(asyncssh.HostKeyNotVerifiable):
                 await self.connect()
 
+    @asynctest
+    async def test_changing_server_host_key(self):
+        """Test changing server host key"""
+
+        self._server.update(server_host_keys=['skey_ecdsa'])
+
+        async with self.connect(known_hosts=None):
+            pass
+
+        self._server.update(server_host_keys=['skey'])
+
+        with self.assertRaises(asyncssh.KeyExchangeFailed):
+            await self.connect(known_hosts=(['skey_ecdsa.pub'], [], []))
+
     @asynctest
     async def test_kex_algs(self):
         """Test connecting with different key exchange algorithms"""
@@ -769,7 +814,17 @@ class _TestConnection(ServerTestCase):
         """Test connecting with invalid key exchange algorithm pattern"""
 
         with self.assertRaises(ValueError):
-            await self.connect(kex_algs='xxx')
+            await self.connect(kex_algs='diffie-hallman-group14-sha1,xxx')
+
+    @asynctest
+    async def test_invalid_kex_alg_config(self):
+        """Test connecting with invalid key exchange algorithm config"""
+
+        with open('config', 'w') as f:
+            f.write('KexAlgorithms diffie-hellman-group14-sha1,xxx')
+
+        async with self.connect(config='config'):
+            pass
 
     @asynctest
     async def test_unsupported_kex_alg(self):
@@ -1180,6 +1235,17 @@ class _TestConnection(ServerTestCase):
 
         await conn.wait_closed()
 
+    @asynctest
+    async def test_no_local_username(self):
+        """Test username being too long in userauth request"""
+
+        def _failing_getuser():
+            raise KeyError
+
+        with patch('getpass.getuser', _failing_getuser):
+            with self.assertRaises(ValueError):
+                await self.connect()
+
     @asynctest
     async def test_invalid_username(self):
         """Test invalid username in userauth request"""
@@ -1191,6 +1257,13 @@ class _TestConnection(ServerTestCase):
 
         await conn.wait_closed()
 
+    @asynctest
+    async def test_username_too_long(self):
+        """Test username being too long in userauth request"""
+
+        with self.assertRaises(asyncssh.IllegalUserName):
+            await self.connect(username=2048*'a')
+
     @asynctest
     async def test_extra_userauth_request(self):
         """Test userauth request after auth is complete"""
@@ -1402,6 +1475,16 @@ class _TestConnectionReverse(ServerTestCase):
             async with self.connect_reverse():
                 pass
 
+    @unittest.skipUnless(_nc_available, 'Netcat not available')
+    @asynctest
+    async def test_connect_reverse_proxy(self):
+        """Test reverse direction SSH connection with proxy command"""
+
+        proxy_command = ('nc', str(self._server_addr), str(self._server_port))
+
+        async with self.connect_reverse(proxy_command=proxy_command):
+            pass
+
     @asynctest
     async def test_connect_reverse_options(self):
         """Test reverse direction SSH connection with options"""
@@ -1759,6 +1842,77 @@ class _TestServerWithoutCert(ServerTestCase):
             pass
 
 
+class _TestHostKeyAlias(ServerTestCase):
+    """Unit test for HostKeyAlias"""
+
+    @classmethod
+    async def start_server(cls):
+        """Start an SSH server to connect to"""
+
+        skey = asyncssh.read_private_key('skey')
+        skey_cert = skey.generate_host_certificate(
+            skey, 'name', principals=['certifiedfakehost'])
+        skey_cert.write_certificate('skey-cert.pub')
+
+        return await cls.create_server(server_host_keys=['skey'])
+
+    @classmethod
+    async def asyncSetUpClass(cls):
+        """Set up keys, custom host cert, and suitable known_hosts"""
+
+        await super().asyncSetUpClass()
+
+        skey_str = Path('skey.pub').read_text()
+
+        Path('.ssh/known_hosts').write_text(
+            f"fakehost {skey_str}"
+            f"@cert-authority certifiedfakehost {skey_str}")
+
+        Path('.ssh/config').write_text(
+            'Host server-with-key-config\n'
+            '  Hostname 127.0.0.1\n'
+            '  HostKeyAlias fakehost\n'
+            '\n'
+            'Host server-with-cert-config\n'
+            '  Hostname 127.0.0.1\n'
+            '  HostKeyAlias certifiedfakehost\n')
+
+    @asynctest
+    async def test_host_key_mismatch(self):
+        """Test host key mismatch"""
+
+        with self.assertRaises(asyncssh.HostKeyNotVerifiable):
+            await self.connect()
+
+    @asynctest
+    async def test_host_key_match(self):
+        """Test host key match"""
+
+        async with self.connect(host_key_alias='fakehost'):
+            pass
+
+    @asynctest
+    async def test_host_cert_match(self):
+        """Test host cert match"""
+
+        async with self.connect(host_key_alias='certifiedfakehost'):
+            pass
+
+    @asynctest
+    async def test_host_key_match_config(self):
+        """Test host key match using HostKeyAlias in config file"""
+
+        async with self.connect('server-with-key-config'):
+            pass
+
+    @asynctest
+    async def test_host_cert_match_config(self):
+        """Test host cert match using HostKeyAlias in config file"""
+
+        async with self.connect('server-with-cert-config'):
+            pass
+
+
 class _TestServerInternalError(ServerTestCase):
     """Unit test for server internal error during auth"""
 
