@@ -1,4 +1,4 @@
-# Copyright (c) 2016-2019 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2016-2020 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -27,7 +27,7 @@ import unittest
 from unittest.mock import patch
 
 import asyncssh
-from asyncssh.misc import async_context_manager
+from asyncssh.misc import async_context_manager, write_file
 from asyncssh.packet import String
 from asyncssh.public_key import CERT_TYPE_USER, CERT_TYPE_HOST
 
@@ -966,6 +966,53 @@ class _TestPublicKeyAuth(ServerTestCase):
         async with self.connect(username='ckey'):
             pass
 
+    @asynctest
+    async def test_agent_identities(self):
+        """Test connecting with ssh-agent auth with specific identities"""
+
+        if not self.agent_available(): # pragma: no cover
+            self.skipTest('ssh-agent not available')
+
+        ckey = asyncssh.read_private_key('ckey')
+        ckey.write_private_key('ckey.pem', 'pkcs8-pem')
+
+        ckey_cert = asyncssh.read_certificate('ckey-cert.pub')
+        ckey_ecdsa = asyncssh.read_public_key('ckey_ecdsa.pub')
+
+        for pubkey in ('ckey-cert.pub', 'ckey_ecdsa.pub', 'ckey.pem',
+                       ckey_cert, ckey_ecdsa, ckey_ecdsa.public_data):
+            async with self.connect(username='ckey', agent_identities=pubkey):
+                pass
+
+    @asynctest
+    async def test_agent_identities_config(self):
+        """Test connecting with ssh-agent auth and IdentitiesOnly config"""
+
+        if not self.agent_available(): # pragma: no cover
+            self.skipTest('ssh-agent not available')
+
+        write_file('ckey_err', b'')
+
+        write_file('config', 'IdentitiesOnly True\n'
+                   'IdentityFile ckey-cert.pub\n'
+                   'IdentityFile ckey_ecdsa.pub\n'
+                   'IdentityFile ckey_err\n', 'w')
+
+        async with self.connect(username='ckey', config='config'):
+            pass
+
+    @asynctest
+    async def test_agent_identities_config_default_keys(self):
+        """Test connecting with ssh-agent auth and default IdentitiesOnly"""
+
+        if not self.agent_available(): # pragma: no cover
+            self.skipTest('ssh-agent not available')
+
+        write_file('config', 'IdentitiesOnly True\n', 'w')
+
+        async with self.connect(username='ckey', config='config'):
+            pass
+
     @asynctest
     async def test_agent_signature_algs(self):
         """Test ssh-agent keys with specific signature algorithms"""
@@ -1107,12 +1154,37 @@ class _TestPublicKeyAuth(ServerTestCase):
                 async with self.connect(username='ckey', client_keys=[key]):
                     pass
 
+    @asynctest
+    async def test_keypair_with_replaced_cert(self):
+        """Test connecting with a keypair with replaced cert"""
+
+        ckey = asyncssh.load_keypairs(['ckey'])[0]
+
+        async with self.connect(username='ckey',
+                                client_keys=[(ckey, 'ckey-cert.pub')]):
+            pass
+
+    @asynctest
+    async def test_agent_keypair_with_replaced_cert(self):
+        """Test connecting with sn agent key with replaced cert"""
+
+        if not self.agent_available(): # pragma: no cover
+            self.skipTest('ssh-agent not available')
+
+        async with asyncssh.connect_agent() as agent:
+            ckey = (await agent.get_keys())[2]
+
+            async with self.connect(username='ckey',
+                                    client_keys=[(ckey, 'ckey-cert.pub')]):
+                pass
+
     @asynctest
     async def test_untrusted_client_key(self):
         """Test untrusted client key"""
 
         with self.assertRaises(asyncssh.PermissionDenied):
-            await self.connect(username='ckey', client_keys='skey')
+            await self.connect(username='ckey', client_keys='skey',
+                               agent_path=None)
 
     @asynctest
     async def test_missing_cert(self):
@@ -1133,7 +1205,8 @@ class _TestPublicKeyAuth(ServerTestCase):
                                 valid_before=1)
 
         with self.assertRaises(asyncssh.PermissionDenied):
-            await self.connect(username='ckey', client_keys=[(skey, cert)])
+            await self.connect(username='ckey', client_keys=[(skey, cert)],
+                               agent_path=None)
 
     @asynctest
     async def test_allowed_address(self):
@@ -1162,7 +1235,8 @@ class _TestPublicKeyAuth(ServerTestCase):
                                 options={'source-address': String('0.0.0.0')})
 
         with self.assertRaises(asyncssh.PermissionDenied):
-            await self.connect(username='ckey', client_keys=[(skey, cert)])
+            await self.connect(username='ckey', client_keys=[(skey, cert)],
+                               agent_path=None)
 
     @asynctest
     async def test_untrusted_ca(self):
@@ -1174,7 +1248,8 @@ class _TestPublicKeyAuth(ServerTestCase):
                                 CERT_TYPE_USER, skey, skey, ['skey'])
 
         with self.assertRaises(asyncssh.PermissionDenied):
-            await self.connect(username='ckey', client_keys=[(skey, cert)])
+            await self.connect(username='ckey', client_keys=[(skey, cert)],
+                               agent_path=None)
 
     @asynctest
     async def test_mismatched_ca(self):
@@ -1378,6 +1453,30 @@ class _TestX509Auth(ServerTestCase):
                                 client_keys=['ckey_x509_chain']):
             pass
 
+    @asynctest
+    async def test_keypair_with_x509_cert(self):
+        """Test connecting with a keypair with replaced X.509 cert"""
+
+        ckey = asyncssh.load_keypairs(['ckey'])[0]
+
+        async with self.connect(username='ckey',
+                                client_keys=[(ckey, 'ckey_x509_chain')]):
+            pass
+
+    @asynctest
+    async def test_agent_keypair_with_x509_cert(self):
+        """Test connecting with sn agent key with replaced X.509 cert"""
+
+        if not self.agent_available(): # pragma: no cover
+            self.skipTest('ssh-agent not available')
+
+        async with asyncssh.connect_agent() as agent:
+            ckey = (await agent.get_keys())[2]
+
+            async with self.connect(username='ckey',
+                                    client_keys=[(ckey, 'ckey_x509_chain')]):
+                pass
+
     @asynctest
     async def test_x509_incomplete_chain(self):
         """Test connecting with incomplete X.509 certificate chain"""
