@@ -1,4 +1,4 @@
-# Copyright (c) 2016-2019 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2016-2020 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -424,8 +424,8 @@ class _TestTCPForwarding(_CheckForwarding):
                                 CERT_TYPE_USER, ckey, ckey, ['ckey'],
                                 extensions={'no-port-forwarding': ''})
 
-        async with self.connect(username='ckey',
-                                client_keys=[(ckey, cert)]) as conn:
+        async with self.connect(username='ckey', client_keys=[(ckey, cert)],
+                                agent_path=None) as conn:
             with self.assertRaises(asyncssh.ChannelOpenError):
                 await conn.open_connection('', 7)
 
@@ -433,8 +433,8 @@ class _TestTCPForwarding(_CheckForwarding):
     async def test_connection_not_permitted_open(self):
         """Test open permission denied in opening a remote connection"""
 
-        async with self.connect(username='ckey',
-                                client_keys=['ckey']) as conn:
+        async with self.connect(username='ckey', client_keys=['ckey'],
+                                agent_path=None) as conn:
             with self.assertRaises(asyncssh.ChannelOpenError):
                 await conn.open_connection('fail', 7)
 
@@ -659,8 +659,8 @@ class _TestTCPForwarding(_CheckForwarding):
                                 CERT_TYPE_USER, ckey, ckey, ['ckey'],
                                 extensions={'no-port-forwarding': ''})
 
-        async with self.connect(username='ckey',
-                                client_keys=[(ckey, cert)]) as conn:
+        async with self.connect(username='ckey', client_keys=[(ckey, cert)],
+                                agent_path=None) as conn:
             with self.assertRaises(asyncssh.ChannelListenError):
                 await conn.forward_remote_port('', 0, '', 0)
 
@@ -797,8 +797,8 @@ class _TestUNIXForwarding(_CheckForwarding):
                                 CERT_TYPE_USER, ckey, ckey, ['ckey'],
                                 extensions={'no-port-forwarding': ''})
 
-        async with self.connect(username='ckey',
-                                client_keys=[(ckey, cert)]) as conn:
+        async with self.connect(username='ckey', client_keys=[(ckey, cert)],
+                                agent_path=None) as conn:
             with self.assertRaises(asyncssh.ChannelOpenError):
                 await conn.open_unix_connection('/echo')
 
@@ -921,8 +921,8 @@ class _TestUNIXForwarding(_CheckForwarding):
                                 CERT_TYPE_USER, ckey, ckey, ['ckey'],
                                 extensions={'no-port-forwarding': ''})
 
-        async with self.connect(username='ckey',
-                                client_keys=[(ckey, cert)]) as conn:
+        async with self.connect(username='ckey', client_keys=[(ckey, cert)],
+                                agent_path=None) as conn:
             with self.assertRaises(asyncssh.ChannelListenError):
                 await conn.forward_remote_path('', 'local')
 
@@ -1024,8 +1024,6 @@ class _TestSOCKSForwarding(_CheckForwarding):
     async def test_forward_socks(self):
         """Test dynamic port forwarding via SOCKS"""
 
-        # pylint: disable=bad-whitespace
-
         _socks_early_errors = [
             ('Bad version',               '0000'),
             ('Bad SOCKSv4 command',       '0400'),
@@ -1053,8 +1051,6 @@ class _TestSOCKSForwarding(_CheckForwarding):
             ('Rejected',    '05010003000001',                   False)
         ]
 
-        # pylint: enable=bad-whitespace
-
         async with self.connect() as conn:
             async with  conn.forward_socks('', 0) as listener:
                 listen_port = listener.get_port()
